### -*-python-*-
###
### Configuration handling
###
### (c) 2013 Mark Wooding
###

###----- Licensing notice ---------------------------------------------------
###
### This file is part of Chopwood: a password-changing service.
###
### Chopwood is free software; you can redistribute it and/or modify
### it under the terms of the GNU Affero General Public License as
### published by the Free Software Foundation; either version 3 of the
### License, or (at your option) any later version.
###
### Chopwood is distributed in the hope that it will be useful,
### but WITHOUT ANY WARRANTY; without even the implied warranty of
### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
### GNU Affero General Public License for more details.
###
### You should have received a copy of the GNU Affero General Public
### License along with Chopwood; if not, see
### <http://www.gnu.org/licenses/>.

from __future__ import with_statement

import os as OS; ENV = OS.environ
import sys as SYS
import types as TY

from auto import PACKAGE, VERSION

### Configuration is done by interpreting a file of Python code.  We expect
### the code to define a number of variables in its global scope.  We import
### a number of identifiers (named in the `EXPORT' list) to this module, and
### also our entire parent module as `chpwd'.

## Names which ought to be exported to the configuration module.
_EXPORT = {}

## The configuration module.
CFG = TY.ModuleType('chpwd_config')

## A list of hooks to call once configuration is complete.
_HOOKS = []
def hook(func):
  """Decorator for post-configuration hooks."""
  _HOOKS.append(func)
  return func

## A suitable set of defaults.
DEFAULTS = {}

def export(*names, **kw):
  """
  Export the names to the configuration module from the caller's environment.
  """

  ## Find the caller's global environment.  Please don't try this at home.
  try: raise Exception
  except: tb = SYS.exc_info()[2]
  env = tb.tb_frame.f_back.f_globals

  ## Export things.
  for name in names:
    _EXPORT[name] = env[name]
  _EXPORT.update(kw)

## Some things to export for sure.
export('PACKAGE', 'VERSION', 'ENV', CONF = SYS.modules[__name__])

def loadconfig(config):
  """
  Load the configuration, populating the `CFG' module with settings.
  """

  ## Make a new module for the configuration, and import ourselves into it.
  d = CFG.__dict__
  d.update(_EXPORT)
  d.update(DEFAULTS)

  ## And run the configuration code.
  with open(config) as f:
    exec f in d

  ## Run the hooks.
  for func in _HOOKS:
    func()

###----- That's all, folks --------------------------------------------------
